@@ -57,10 +57,11 @@
  * that we resort to using thread local storage instead of having
  * per-client data.
  */
-#if LIBVNCSERVER_HAVE_LIBPTHREAD && LIBVNCSERVER_HAVE_TLS && !defined(TLS) && defined(__linux__)
+#if defined(__GNUC__)
 #define TLS __thread
-#endif
-#ifndef TLS
+#elif defined(_MSC_VER)
+#define TLS __declspec(thread)
+#else
 #define TLS
 #endif
 
@@ -163,7 +164,11 @@ void rfbTightCleanup (rfbScreenInfoPtr screen)
         tightAfterBufSize = 0;
         tightAfterBuf = NULL;
     }
-    if (j) tjDestroy(j);
+	if (j) {
+		tjDestroy(j);
+		/* Set freed resource handle to 0! */
+		j = 0;
+	}
 }
 
 
@@ -187,7 +192,6 @@ static rfbBool CheckSolidTile32  (rfbClientPtr cl, int x, int y, int w, int h,
 
 static rfbBool SendRectSimple    (rfbClientPtr cl, int x, int y, int w, int h);
 static rfbBool SendSubrect       (rfbClientPtr cl, int x, int y, int w, int h);
-static rfbBool SendTightHeader   (rfbClientPtr cl, int x, int y, int w, int h);
 
 static rfbBool SendSolidRect     (rfbClientPtr cl);
 static rfbBool SendMonoRect      (rfbClientPtr cl, int x, int y, int w, int h);
@@ -196,8 +200,6 @@ static rfbBool SendFullColorRect (rfbClientPtr cl, int x, int y, int w, int h);
 
 static rfbBool CompressData (rfbClientPtr cl, int streamId, int dataLen,
                              int zlibLevel, int zlibStrategy);
-static rfbBool SendCompressedData (rfbClientPtr cl, char *buf,
-                                   int compressedLen);
 
 static void FillPalette8 (int count);
 static void FillPalette16 (int count);
@@ -426,7 +428,7 @@ SendRectEncodingTight(rfbClientPtr cl,
 
                 /* Send solid-color rectangle. */
 
-                if (!SendTightHeader(cl, x_best, y_best, w_best, h_best))
+                if (!rfbSendTightHeader(cl, x_best, y_best, w_best, h_best))
                     return FALSE;
 
                 fbptr = (cl->scaledScreen->frameBuffer +
@@ -559,7 +561,7 @@ ExtendSolidArea(rfbClientPtr cl,
 /*
  * Check if a rectangle is all of the same color. If needSameColor is
  * set to non-zero, then also check that its color equals to the
- * *colorPtr value. The result is 1 if the test is successfull, and in
+ * *colorPtr value. The result is 1 if the test is successful, and in
  * that case new color will be stored in *colorPtr.
  */
 
@@ -679,7 +681,7 @@ SendSubrect(rfbClientPtr cl,
             return FALSE;
     }
 
-    if (!SendTightHeader(cl, x, y, w, h))
+    if (!rfbSendTightHeader(cl, x, y, w, h))
         return FALSE;
 
     fbptr = (cl->scaledScreen->frameBuffer
@@ -763,8 +765,8 @@ SendSubrect(rfbClientPtr cl,
     return success;
 }
 
-static rfbBool
-SendTightHeader(rfbClientPtr cl,
+rfbBool
+rfbSendTightHeader(rfbClientPtr cl,
                 int x,
                 int y,
                 int w,
@@ -1040,7 +1042,7 @@ CompressData(rfbClientPtr cl,
     }
 
     if (zlibLevel == 0)
-        return SendCompressedData (cl, tightBeforeBuf, dataLen);
+        return rfbSendCompressedDataTight(cl, tightBeforeBuf, dataLen);
 
     pz = &cl->zsStruct[streamId];
 
@@ -1079,12 +1081,12 @@ CompressData(rfbClientPtr cl,
         return FALSE;
     }
 
-    return SendCompressedData(cl, tightAfterBuf,
-                              tightAfterBufSize - pz->avail_out);
+    return rfbSendCompressedDataTight(cl, tightAfterBuf,
+                                      tightAfterBufSize - pz->avail_out);
 }
 
-static rfbBool SendCompressedData(rfbClientPtr cl, char *buf,
-                                  int compressedLen)
+rfbBool rfbSendCompressedDataTight(rfbClientPtr cl, char *buf,
+                                   int compressedLen)
 {
     int i, portionLen;
 
@@ -1661,7 +1663,7 @@ SendJpegRect(rfbClientPtr cl, int x, int y, int w, int h, int quality)
     cl->updateBuf[cl->ublen++] = (char)(rfbTightJpeg << 4);
     rfbStatRecordEncodingSentAdd(cl, cl->tightEncoding, 1);
 
-    return SendCompressedData(cl, tightAfterBuf, (int)size);
+    return rfbSendCompressedDataTight(cl, tightAfterBuf, (int)size);
 }
 
 static void
@@ -1895,6 +1897,6 @@ static rfbBool SendPngRect(rfbClientPtr cl, int x, int y, int w, int h) {
     rfbStatRecordEncodingSentAdd(cl, cl->tightEncoding, 1);
 
     /* rfbLog("<< SendPngRect\n"); */
-    return SendCompressedData(cl, tightAfterBuf, pngDstDataLen);
+    return rfbSendCompressedDataTight(cl, tightAfterBuf, pngDstDataLen);
 }
 #endif
